/******************************************************************************

	FILE				: sci.c
   
	AUTHOR			: Axiom Manufacturing    					     
						  Spencer Ruggles
						  
	PURPOSE			: Manufacturing Test for CSM12C128 board, x436
 
 *****************************************************************************/
 

#include "MC9RS08LA8.h"				// include peripheral declarations 
#include "sci.h"							// sci header file

	

/*******************************************************************************
ensci()

	configures COM port selected by (port) for 19200,8,N,1 operation
	SCI baud rate register (SCIxBD) is written with value determined by 
	     SCIxBD = busclk/(16*baud) => SCIxBD = 0x0D -> 19.2 kHz
	     
	MCG trim value must be written to effectively use COM port 
	
	INPUT:	none
	OUTPUT:	none
********************************************************************************/
void ensci(void){
		SCIBD = baud19;										// 19.2 kHz
		SCIC1 &= ~SCIC1_M_MASK & ~SCIC1_PE_MASK;	// 8,N,1
		SCIC2 |= SCIC2_TE_MASK | SCIC2_RE_MASK;	// SCI Tx/Rx enabled
}

/*******************************************************************************
ReadyToSend

	wait for selected COM port Tx Data Reg Empty flag to set indicating tx is 
	ready for a byte of data
	
	INPUT:	none
	OUTPUT:	returns 	0 when ready - no TIMEOUE feature implemented
********************************************************************************/
byte ReadyToSend(void) {
	byte status;							// temp var
	
	do {
		status = SCIS1;					// get COM1 TDRE flag status
	} while((status & 0x80) == 0);	// loop until TDRE flag is set
	return 0;								// return 0 
}

/*******************************************************************************
putchar
	check for ReadyToSend then output byte of data on selected COM port
	
	INPUT:	sbyte 	output data to write to COM port
	OUTPUT:	returns 	0 when ready - no TIMEOUE feature implemented
********************************************************************************/
byte putchar(byte sbyte) {
	if(ReadyToSend()) return 1;		// wait for TDRE flag to set
	
	SCID = sbyte;							// send data byte on COM1
	while(!SCIS1_TC);						// wait for Tx Complete flag to set
	return 0;								// return successful
}

/*******************************************************************************
putch

	send a single byte of data out selected COM port.  if input byte is '\n' char
	then send carriage return and line feed (CR\LF)
	
	INPUT:	sbyte 	output data to write to COM port
	OUTPUT: none
********************************************************************************/
void putch(byte ch){
	if(ch == '\n'){	 				// if \n send CR/LF
		putchar(CR);
		putchar(LF);
	} else {
		putchar(ch);					// send data byte
	}
}

/*******************************************************************************
puts

	send a string of data out selected COM port with CR/LF.  string should be null 	
	terminated
	
	INPUT:		sptr		pointer to data string
	OUTPUT: none
*******************************************************************************
void puts(byte *sptr) {
	while(*sptr) {						// if data still in string
		putch(*sptr);			// send data to selected COM port w/ CR/LF
		++sptr;
	}
}*/

/*******************************************************************************
fputs

	send a string of data out selected COM port with CR/LF.  string should be null 	
	terminated
	
	INPUT:	sptr		pointer to data string
	OUTPUT:	none
*******************************************************************************
void fputs(byte *sptr) {
	byte ch;								
	while(*sptr) {						// if data still in string
		ch = *sptr;						// get data byte
//		if(ch="\n") break;
		putchar(ch);					// send data
//		putch(port, *sptr);			// send data to selected COM port w/ CR/LF
		++sptr;
	}
}*/

/*******************************************************************************
SerialReady

	returns 0 if new serial data avaiable on selected COM port
	
	INPUT:		
	OUTPUT: 		status	0 = data available,  1 = no data avaiable
********************************************************************************/
byte SerialReady(void){
	byte status;						// temp variable
	
	status = SCIS1;			// get status on COM1

	if(status & 0x20) return 0;	// return 0 if new data available
	else return 1;						// return 1 if no data available
}

/*******************************************************************************
ReadSerialPort

	reads a single byte from selected serial port
	
	INPUT:		none
	OUTPUT: 		returns the data read from selected COM port 
********************************************************************************/
byte ReadSerialPort(void){
	byte sbyte;							// temp var
	
	sbyte = SCID;						// get COM1 data
	
	return sbyte;						// return data byte read
}

/*******************************************************************************
GetSByte

	wait for Rx Data Ready Flag (RDRF) to set calls ReadSerialPort to get data
	from selected serial port
	
	INPUT:		none
	OUTPUT: 		returns the data read from selected COM port 
********************************************************************************/
byte GetSByte(void){
	
	if(SerialReady()){					// if data available
		return(ReadSerialPort());		// read input data on selected COM port
	} else return 0;						// no data available
}

/*******************************************************************************
hex2char


	input: 4-bit hex value in low nibble of input
	outpt: ASCII representation of input
********************************************************************************/
byte hex2char(byte hval) {
	if(hval < 0x0A) {
		hval += 0x30;						// convert high nibble numeric to ASCII text 
	} else {
		hval += 0x37;						// convert high nibble alpha to ASCII text
	}
	return hval;							// return ASCII char
}

/*******************************************************************************
char2hex

	converts an input  value to its hex value.  ONLY CONVERTS ASCII NUMBERS. 
	
	input: 8-bit ASCII char 
	outpt: hex byte
********************************************************************************/
byte char2hex(byte cval){
	if(cval > 0x30 && cval < 0x3A){
		cval -= 0x30;						// convert char to hex
	}
	return cval;							// return hex val
}

/*******************************************************************************
outhex

	formats 8-bit hex value into ASCII for display on selected COM port
	
	input: 8-bit hex value 
	outpt: ASCII representation of input to selected COM port
********************************************************************************/
void outhex(byte hval){
	// convert input HEX value to output ASCII text

	unsigned char unib,lnib;			// temp variable for upper and lower nibbles
	
	unib = (hval >> 4);					// high nibble
	lnib = hval & 0x0F;					// low nibble

	putch(hex2char(unib));		// send char out selected COM port
	putch(hex2char(lnib));		// send char out selected COM port
}  

